@@ -19,17 +19,31 @@ class COSE(CBORProcessor):
         """
         Constructor.
 
-        At the current implementation, any ``options`` will be ignored.
+        Args:
+            options (Optional[Dict[str, Any]]): Options for the initial configuration
+                of COSE. At this time, ``kid_auto_inclusion`` (default value: ``True``)
+                and ``alg_auto_inclusion`` (default value: ``True``) are supported.
         """
-        self._options = options
         self._recipients_builder = RecipientsBuilder()
+        self._kid_auto_inclusion = True
+        self._alg_auto_inclusion = True
+        if not options:
+            return
+        if "kid_auto_inclusion" in options:
+            if not isinstance(options["kid_auto_inclusion"], bool):
+                raise ValueError("kid_auto_inclusion should be bool.")
+            self._kid_auto_inclusion = options["kid_auto_inclusion"]
+        if "alg_auto_inclusion" in options:
+            if not isinstance(options["alg_auto_inclusion"], bool):
+                raise ValueError("alg_auto_inclusion should be bool.")
+            self._alg_auto_inclusion = options["alg_auto_inclusion"]
 
     def encode_and_mac(
         self,
-        protected: Dict[int, Any],
-        unprotected: Dict[int, Any],
-        payload: Union[Dict[int, Any], bytes],
+        payload: bytes,
         key: COSEKey,
+        protected: Union[Dict[int, Any], bytes] = {},
+        unprotected: Dict[int, Any] = {},
         recipients: Optional[List[Recipient]] = None,
         out: str = "",
     ) -> Union[bytes, CBORTag]:
@@ -37,12 +51,11 @@ class COSE(CBORProcessor):
         Encodes data with MAC.
 
         Args:
-            protected (Dict[int, Any]): Parameters that are to be cryptographically
-                protected.
-            unprotected (Dict[int, Any]): Parameters that are not cryptographically
-                protected.
-            payload (Union[Dict[int, Any], bytes]): A content to be MACed.
+            payload (bytes): A content to be MACed.
             key (COSEKey): A COSE key as a MAC Authentication key.
+            protected (Union[Dict[int, Any], bytes]): Parameters that are to be cryptographically
+                protected.
+            unprotected (Dict[int, Any]): Parameters that are not cryptographically protected.
             recipients (Optional[List[Recipient]]): A list of recipient information structures.
             out(str): An output format. Only ``"cbor2/CBORTag"`` can be used. If ``"cbor2/CBORTag"``
                 is specified. This function will return encoded data as
@@ -56,50 +69,67 @@ class COSE(CBORProcessor):
         """
 
         ctx = "MAC0" if not recipients else "MAC"
+        b_protected = b""
 
         # MAC0
         if not recipients:
-            protected[1] = key.alg
-            if key.kid:
-                unprotected[4] = key.kid
-            b_protected = self._dumps(protected)
-            b_payload = self._dumps(payload)
-            mac_structure = [ctx, b_protected, b"", b_payload]
+            if isinstance(protected, bytes):
+                b_protected = protected
+            else:
+                if self._alg_auto_inclusion:
+                    protected[1] = key.alg
+                if self._kid_auto_inclusion and key.kid:
+                    unprotected[4] = key.kid
+                b_protected = self._dumps(protected)
+            mac_structure = [ctx, b_protected, b"", payload]
             tag = key.sign(self._dumps(mac_structure))
-            res = CBORTag(17, [b_protected, unprotected, b_payload, tag])
+            res = CBORTag(17, [b_protected, unprotected, payload, tag])
             return res if out == "cbor2/CBORTag" else self._dumps(res)
 
         # MAC
-        b_protected = self._dumps(protected) if protected else b""
-        b_payload = self._dumps(payload)
-        mac_structure = [ctx, b_protected, b"", b_payload]
-        tag = key.sign(self._dumps(mac_structure))
-        cose_mac: List[Any] = [b_protected, unprotected, b_payload, tag]
         recs = []
         for rec in recipients:
             recs.append(rec.to_list())
+        if recipients[0].alg == -6:
+            if not isinstance(protected, bytes):
+                if self._alg_auto_inclusion:
+                    protected[1] = key.alg
+                if self._kid_auto_inclusion and key.kid:
+                    unprotected[4] = key.kid
+        else:
+            raise NotImplementedError(
+                "Algorithms other than direct are not supported for recipients."
+            )
+
+        if isinstance(protected, bytes):
+            b_protected = protected
+        else:
+            b_protected = self._dumps(protected) if protected else b""
+        mac_structure = [ctx, b_protected, b"", payload]
+        tag = key.sign(self._dumps(mac_structure))
+        cose_mac: List[Any] = [b_protected, unprotected, payload, tag]
         cose_mac.append(recs)
         res = CBORTag(97, cose_mac)
         return res if out == "cbor2/CBORTag" else self._dumps(res)
 
     def encode_and_sign(
         self,
-        protected: Dict[int, Any],
-        unprotected: Dict[int, Any],
-        payload: Union[Dict[int, Any], bytes],
+        payload: bytes,
         key: Union[COSEKey, List[COSEKey]],
+        protected: Union[Dict[int, Any], bytes] = {},
+        unprotected: Dict[int, Any] = {},
         out: str = "",
     ) -> Union[bytes, CBORTag]:
         """
         Encodes data with signing.
 
         Args:
-            protected (Dict[int, Any]): Parameters that are to be cryptographically
+            payload (bytes): A content to be signed.
+            key (Union[COSEKey, List[COSEKey]]): One or more COSE keys as signing keys.
+            protected (Union[Dict[int, Any], bytes]): Parameters that are to be cryptographically
                 protected.
             unprotected (Dict[int, Any]): Parameters that are not cryptographically
                 protected.
-            payload (Union[Dict[int, Any], bytes]): A content to be signed.
-            key (Union[COSEKey, List[COSEKey]]): One or more COSE keys as signing keys.
             out(str): An output format. Only ``"cbor2/CBORTag"`` can be used. If ``"cbor2/CBORTag"``
                 is specified. This function will return encoded data as
                 `cbor2 <https://cbor2.readthedocs.io/en/stable/>`_'s ``CBORTag`` object.
@@ -112,38 +142,43 @@ class COSE(CBORProcessor):
         """
 
         ctx = "Signature" if not isinstance(key, COSEKey) else "Signature1"
-        if isinstance(key, COSEKey):
-            protected[1] = key.alg
-            if key.kid:
+        if isinstance(key, COSEKey) and isinstance(protected, dict):
+            if self._alg_auto_inclusion:
+                protected[1] = key.alg
+            if self._kid_auto_inclusion and key.kid:
                 unprotected[4] = key.kid
 
-        b_protected = self._dumps(protected) if protected else b""
-        b_payload = self._dumps(payload)
+        b_protected = b""
+        if isinstance(protected, bytes):
+            b_protected = protected
+        else:
+            b_protected = self._dumps(protected) if protected else b""
 
         # Signature1
         if isinstance(key, COSEKey):
-            sig_structure = [ctx, b_protected, b"", b_payload]
+            sig_structure = [ctx, b_protected, b"", payload]
             sig = key.sign(self._dumps(sig_structure))
-            res = CBORTag(18, [b_protected, unprotected, b_payload, sig])
+            res = CBORTag(18, [b_protected, unprotected, payload, sig])
             return res if out == "cbor2/CBORTag" else self._dumps(res)
 
         # Signature
         sigs = []
         for k in key:
+            print(f"alg={k.alg}")
             p_header = self._dumps({1: k.alg})
             u_header = {4: k.kid} if k.kid else {}
-            sig_structure = [ctx, b_protected, p_header, b"", b_payload]
+            sig_structure = [ctx, b_protected, p_header, b"", payload]
             sig = k.sign(self._dumps(sig_structure))
             sigs.append([p_header, u_header, sig])
-        res = CBORTag(98, [b_protected, unprotected, b_payload, sigs])
+        res = CBORTag(98, [b_protected, unprotected, payload, sigs])
         return res if out == "cbor2/CBORTag" else self._dumps(res)
 
     def encode_and_encrypt(
         self,
-        protected: Dict[int, Any],
-        unprotected: Dict[int, Any],
-        payload: Union[Dict[int, Any], bytes],
+        payload: bytes,
         key: COSEKey,
+        protected: Union[Dict[int, Any], bytes] = {},
+        unprotected: Dict[int, Any] = {},
         nonce: bytes = b"",
         recipients: Optional[List[Recipient]] = None,
         out: str = "",
@@ -152,12 +187,12 @@ class COSE(CBORProcessor):
         Encodes data with encryption.
 
         Args:
-            protected (Dict[int, Any]): Parameters that are to be cryptographically
+            payload (bytes): A content to be encrypted.
+            key (COSEKey): A COSE key as an encryption key.
+            protected (Union[Dict[int, Any], bytes]): Parameters that are to be cryptographically
                 protected.
             unprotected (Dict[int, Any]): Parameters that are not cryptographically
                 protected.
-            payload (Union[Dict[int, Any], bytes]): A content to be encrypted.
-            key (COSEKey): A COSE key as an encryption key.
             nonce (bytes): A nonce for encryption.
             recipients (Optional[List[Recipient]]): A list of recipient information structures.
             out(str): An output format. Only ``"cbor2/CBORTag"`` can be used. If ``"cbor2/CBORTag"``
@@ -173,33 +208,60 @@ class COSE(CBORProcessor):
 
         ctx = "Encrypt0" if not recipients else "Encrypt"
 
+        if not nonce:
+            try:
+                nonce = key.generate_nonce()
+            except NotImplementedError:
+                raise ValueError(
+                    "Nonce generation is not supported for the key. Set a nonce explicitly."
+                )
+
         # Encrypt0
         if not recipients:
-            b_protected = self._dumps(protected) if protected else b""
-            b_payload = self._dumps(payload)
+            if isinstance(protected, bytes):
+                b_protected = protected
+            else:
+                if self._alg_auto_inclusion:
+                    protected[1] = key.alg
+                b_protected = self._dumps(protected) if protected else b""
+            if self._kid_auto_inclusion and key.kid:
+                unprotected[4] = key.kid
+            unprotected[5] = nonce
             enc_structure = [ctx, b_protected, b""]
             aad = self._dumps(enc_structure)
-            ciphertext = key.encrypt(b_payload, nonce, aad)
+            ciphertext = key.encrypt(payload, nonce, aad)
             res = CBORTag(16, [b_protected, unprotected, ciphertext])
             return res if out == "cbor2/CBORTag" else self._dumps(res)
 
         # Encrypt
-        b_protected = self._dumps(protected) if protected else b""
-        b_payload = self._dumps(payload)
-        enc_structure = [ctx, b_protected, b""]
-        aad = self._dumps(enc_structure)
-        ciphertext = key.encrypt(b_payload, nonce, aad)
-        cose_enc: List[Any] = [b_protected, unprotected, ciphertext]
         recs = []
         for rec in recipients:
             recs.append(rec.to_list())
+        if recipients[0].alg == -6:
+            if not isinstance(protected, bytes) and self._alg_auto_inclusion:
+                protected[1] = key.alg
+            if self._kid_auto_inclusion and key.kid:
+                unprotected[4] = key.kid
+            unprotected[5] = nonce
+        else:
+            raise NotImplementedError(
+                "Algorithms other than direct are not supported for recipients."
+            )
+        if isinstance(protected, bytes):
+            b_protected = protected
+        else:
+            b_protected = self._dumps(protected) if protected else b""
+        enc_structure = [ctx, b_protected, b""]
+        aad = self._dumps(enc_structure)
+        ciphertext = key.encrypt(payload, nonce, aad)
+        cose_enc: List[Any] = [b_protected, unprotected, ciphertext]
         cose_enc.append(recs)
         res = CBORTag(96, cose_enc)
         return res if out == "cbor2/CBORTag" else self._dumps(res)
 
     def decode(
         self, data: Union[bytes, CBORTag], key: Union[COSEKey, List[COSEKey]]
-    ) -> Dict[int, Any]:
+    ) -> bytes:
         """
         Verifies and decodes COSE data.
 
@@ -208,7 +270,7 @@ class COSE(CBORProcessor):
                 encoded data.
             key (COSEKey): A COSE key to verify and decrypt the encoded data.
         Returns:
-            Dict[int, Any]: A decoded CBOR-like object.
+            bytes: A byte string of decoded payload.
         Raises:
             ValueError: Invalid arguments.
             DecodeError: Failed to decode data.
@@ -235,8 +297,7 @@ class COSE(CBORProcessor):
             k = self._get_key(keys, unprotected)
             if not k:
                 raise ValueError("key is not specified.")
-            payload = k.decrypt(data.value[2], nonce, aad)
-            return self._loads(payload)
+            return k.decrypt(data.value[2], nonce, aad)
 
         # Encrypt
         if data.tag == 96:
@@ -251,8 +312,7 @@ class COSE(CBORProcessor):
             nonce = unprotected.get(5, None)
             recipients = self._recipients_builder.from_list(data.value[3])
             enc_key = recipients.derive_key(keys)
-            payload = enc_key.decrypt(data.value[2], nonce, aad)
-            return self._loads(payload)
+            return enc_key.decrypt(data.value[2], nonce, aad)
 
         # MAC0
         if data.tag == 17:
@@ -265,7 +325,7 @@ class COSE(CBORProcessor):
             if not k:
                 raise ValueError("key is not specified.")
             k.verify(msg, data.value[3])
-            return self._loads(data.value[2])
+            return data.value[2]
 
         # MAC
         if data.tag == 97:
@@ -276,7 +336,7 @@ class COSE(CBORProcessor):
             recipients = self._recipients_builder.from_list(data.value[4])
             mac_auth_key = recipients.derive_key(keys)
             mac_auth_key.verify(to_be_maced, data.value[3])
-            return self._loads(data.value[2])
+            return data.value[2]
 
         # Signature1
         if data.tag == 18:
@@ -291,7 +351,7 @@ class COSE(CBORProcessor):
             if not k:
                 raise ValueError("key is not specified.")
             k.verify(to_be_signed, data.value[3])
-            return self._loads(data.value[2])
+            return data.value[2]
 
         # Signature
         if data.tag == 98:
@@ -311,7 +371,7 @@ class COSE(CBORProcessor):
                     ["Signature", data.value[0], sig[0], b"", data.value[2]]
                 )
                 k.verify(to_be_signed, sig[2])
-                return self._loads(data.value[2])
+                return data.value[2]
             raise ValueError("Verification key not found.")
         raise ValueError(f"Unsupported or unknown CBOR tag({data.tag}).")
 
