@@ -36,7 +36,7 @@ from .const import (
     COSE_ALGORITHMS_SYMMETRIC,
     COSE_KEY_OPERATION_VALUES,
     COSE_KEY_TYPES,
-    JOSE_ALGORITHMS_SUPPORTED,
+    COSE_NAMED_ALGORITHMS_SUPPORTED,
     JWK_ELLIPTIC_CURVES,
     JWK_OPERATIONS,
     JWK_PARAMS_EC,
@@ -221,9 +221,9 @@ class KeyBuilder(CBORProcessor):
         if "alg" in jwk:
             if not isinstance(jwk["alg"], str):
                 raise ValueError("alg should be str.")
-            if jwk["alg"] not in JOSE_ALGORITHMS_SUPPORTED:
+            if jwk["alg"] not in COSE_NAMED_ALGORITHMS_SUPPORTED:
                 raise ValueError(f"Unsupported or unknown alg: {jwk['alg']}.")
-            cose_key[3] = JOSE_ALGORITHMS_SUPPORTED[jwk["alg"]]
+            cose_key[3] = COSE_NAMED_ALGORITHMS_SUPPORTED[jwk["alg"]]
 
         # key operation dependent conversion
         is_public = False
@@ -248,12 +248,24 @@ class KeyBuilder(CBORProcessor):
             cose_key[-1] = JWK_ELLIPTIC_CURVES[jwk["crv"]]
 
             if cose_key[1] == 1:  # OKP
+                if 3 not in cose_key:
+                    cose_key[3] = -8  # EdDSA
                 for k, v in jwk.items():
                     if k not in JWK_PARAMS_OKP:
                         continue
                     cose_key[JWK_PARAMS_OKP[k]] = base64url_decode(v)
 
             else:  # EC2
+                if 3 not in cose_key:
+                    if cose_key[-1] == 1:  # P-256
+                        cose_key[3] = -7
+                    elif cose_key[-1] == 2:  # P-384
+                        cose_key[3] = -35
+                    elif cose_key[-1] == 3:  # P-521
+                        cose_key[3] = -36
+                    else:
+                        # cose_key[-1] == 8  # secp256k1
+                        cose_key[3] = -47
                 for k, v in jwk.items():
                     if k not in JWK_PARAMS_EC:
                         continue
@@ -473,7 +485,7 @@ class KeyBuilder(CBORProcessor):
             DecodeError: Failed to decode the COSE key.
             VerifyError: Failed to verify the COSE key.
         """
-        res = self._cose.decode(CBORTag(16, key), encryption_key)
+        res = self._loads(self._cose.decode(CBORTag(16, key), encryption_key))
         return self.from_dict(res)
 
     def to_encrypted_cose_key(
@@ -508,12 +520,13 @@ class KeyBuilder(CBORProcessor):
                     "Nonce generation is not supported for the key. Set a nonce explicitly."
                 )
         unprotected[5] = nonce
+        b_payload = self._dumps(key.to_dict())
         res: CBORTag = self._cose.encode_and_encrypt(
+            b_payload,
+            encryption_key,
             protected,
             unprotected,
-            key.to_dict(),
-            encryption_key,
-            nonce,
+            nonce=nonce,
             out="cbor2/CBORTag",
         )
         return res.value
